@@ -4,6 +4,11 @@ import boto3
 import click
 
 from newrelic_lambda_cli import api, integrations, permissions
+from newrelic_lambda_cli.types import (
+    IntegrationInstall,
+    IntegrationUninstall,
+    IntegrationUpdate,
+)
 from newrelic_lambda_cli.cli.decorators import add_options, AWS_OPTIONS, NR_OPTIONS
 from newrelic_lambda_cli.cliutils import done, failure
 
@@ -90,79 +95,64 @@ def register(group):
     nargs=2,
 )
 @click.pass_context
-def install(
-    ctx,
-    aws_profile,
-    aws_region,
-    aws_permissions_check,
-    aws_role_policy,
-    enable_logs,
-    memory_size,
-    linked_account_name,
-    nr_account_id,
-    nr_api_key,
-    nr_region,
-    timeout,
-    role_name,
-    enable_license_key_secret,
-    integration_arn,
-    tags,
-):
+def install(ctx, **kwargs):
     """Install New Relic AWS Lambda Integration"""
-    session = boto3.Session(profile_name=aws_profile, region_name=aws_region)
+    input = IntegrationInstall(session=None, verbose=ctx.obj["VERBOSE"], **kwargs)
 
-    if aws_permissions_check:
-        permissions.ensure_integration_install_permissions(session)
+    input = input._replace(
+        session=boto3.Session(
+            profile_name=input.aws_profile, region_name=input.aws_region
+        )
+    )
+
+    if not input.linked_account_name:
+        input = input._replace(
+            linked_account_name=(
+                "New Relic Lambda Integration - %s"
+                % integrations.get_aws_account_id(input.session)
+            )
+        )
+
+    if input.aws_permissions_check:
+        permissions.ensure_integration_install_permissions(input)
 
     click.echo("Validating New Relic credentials")
-    gql_client = api.validate_gql_credentials(nr_account_id, nr_api_key, nr_region)
+    gql_client = api.validate_gql_credentials(input)
 
     click.echo("Retrieving integration license key")
     nr_license_key = api.retrieve_license_key(gql_client)
 
-    if not linked_account_name:
-        linked_account_name = (
-            "New Relic Lambda Integration - %s"
-            % integrations.get_aws_account_id(session)
-        )
-
     click.echo("Checking for a pre-existing link between New Relic and AWS")
-    integrations.validate_linked_account(session, gql_client, linked_account_name)
-
-    click.echo("Creating the AWS role for the New Relic AWS Lambda Integration")
-    role = integrations.create_integration_role(
-        session, aws_role_policy, nr_account_id, integration_arn, tags
-    )
-
-    if enable_license_key_secret:
-        click.echo("Creating the managed secret for the New Relic License Key")
-        integrations.install_license_key(session, nr_license_key, tags=tags)
+    integrations.validate_linked_account(gql_client, input)
 
     install_success = True
 
+    click.echo("Creating the AWS role for the New Relic AWS Lambda Integration")
+    role = integrations.create_integration_role(input)
+    install_success = install_success and role
+
     if role:
         click.echo("Linking New Relic account to AWS account")
-        res = api.create_integration_account(
-            gql_client, nr_account_id, linked_account_name, role
-        )
+        res = api.create_integration_account(gql_client, input, role)
         install_success = res and install_success
 
         click.echo("Enabling Lambda integration on the link between New Relic and AWS")
-        res = api.enable_lambda_integration(
-            gql_client, nr_account_id, linked_account_name
-        )
+        res = api.enable_lambda_integration(gql_client, input)
         install_success = res and install_success
 
+    if input.enable_license_key_secret:
+        click.echo("Creating the managed secret for the New Relic License Key")
+        res = integrations.install_license_key(input, nr_license_key)
+        install_success = install_success and res
+
     click.echo("Creating newrelic-log-ingestion Lambda function in AWS account")
-    res = integrations.install_log_ingestion(
-        session, nr_license_key, enable_logs, memory_size, timeout, role_name, tags
-    )
+    res = integrations.install_log_ingestion(input, nr_license_key)
     install_success = res and install_success
 
     if install_success:
         done("Install Complete")
 
-        if ctx.obj["VERBOSE"]:
+        if input.verbose:
             click.echo(
                 "\nNext steps: Add the New Relic layers to your Lambda functions with "
                 "the below command.\n"
@@ -175,12 +165,12 @@ def install(
                 "--function",
                 "all",
                 "--nr-account-id",
-                nr_account_id,
+                input.nr_account_id,
             ]
-            if aws_profile:
-                command.append("--aws-profile %s" % aws_profile)
-            if aws_region:
-                command.append("--aws-region %s" % aws_region)
+            if input.aws_profile:
+                command.append("--aws-profile %s" % input.aws_profile)
+            if input.aws_region:
+                command.append("--aws-region %s" % input.aws_region)
             click.echo(" ".join(command))
     else:
         failure("Install Incomplete. See messages above for details.", exit=True)
@@ -198,25 +188,31 @@ def install(
     type=click.INT,
 )
 @click.option("--force", "-f", help="Force uninstall non-interactively", is_flag=True)
-def uninstall(aws_profile, aws_region, aws_permissions_check, nr_account_id, force):
+def uninstall(**kwargs):
     """Uninstall New Relic AWS Lambda Integration"""
-    session = boto3.Session(profile_name=aws_profile, region_name=aws_region)
+    input = IntegrationUninstall(session=None, **kwargs)
 
-    if aws_permissions_check:
-        permissions.ensure_integration_uninstall_permissions(session)
+    input = input._replace(
+        session=boto3.Session(
+            profile_name=input.aws_profile, region_name=input.aws_region
+        )
+    )
+
+    if input.aws_permissions_check:
+        permissions.ensure_integration_uninstall_permissions(input)
 
     uninstall_integration = True
 
-    if not force and nr_account_id:
+    if not input.force and input.nr_account_id:
         uninstall_integration = click.confirm(
             "This will uninstall the New Relic AWS Lambda integration role. "
             "Are you sure you want to proceed?"
         )
 
-    if uninstall_integration and nr_account_id:
-        integrations.remove_integration_role(session, nr_account_id)
+    if uninstall_integration and input.nr_account_id:
+        integrations.remove_integration_role(input)
 
-    if not force:
+    if not input.force:
         click.confirm(
             "This will uninstall the New Relic AWS Lambda log ingestion function and "
             "role. Are you sure you want to proceed?",
@@ -224,9 +220,9 @@ def uninstall(aws_profile, aws_region, aws_permissions_check, nr_account_id, for
             default=False,
         )
 
-    integrations.remove_log_ingestion_function(session)
+    integrations.remove_log_ingestion_function(input)
 
-    if not force:
+    if not input.force:
         click.confirm(
             "This will uninstall the New Relic License Key managed secret, and IAM "
             "Policy. "
@@ -234,7 +230,7 @@ def uninstall(aws_profile, aws_region, aws_permissions_check, nr_account_id, for
             abort=True,
             default=False,
         )
-    integrations.remove_license_key(session)
+    integrations.remove_license_key(input)
 
     done("Uninstall Complete")
 
@@ -282,37 +278,29 @@ def uninstall(aws_profile, aws_region, aws_permissions_check, nr_account_id, for
     multiple=True,
     nargs=2,
 )
-def update(
-    aws_profile,
-    aws_region,
-    aws_permissions_check,
-    enable_logs,
-    memory_size,
-    timeout,
-    role_name,
-    enable_license_key_secret,
-    tags,
-):
+def update(**kwargs):
     """UpdateNew Relic AWS Lambda Integration"""
-    session = boto3.Session(profile_name=aws_profile, region_name=aws_region)
+    input = IntegrationUpdate(session=None, **kwargs)
 
-    if aws_permissions_check:
-        permissions.ensure_integration_install_permissions(session)
+    input = input._replace(
+        session=boto3.Session(
+            profile_name=input.aws_profile, region_name=input.aws_region
+        )
+    )
+
+    if input.aws_permissions_check:
+        permissions.ensure_integration_install_permissions(input)
 
     update_success = True
 
     click.echo("Updating newrelic-log-ingestion Lambda function in AWS account")
-    res = integrations.update_log_ingestion(
-        session, None, enable_logs, memory_size, timeout, role_name, tags
-    )
+    res = integrations.update_log_ingestion(input)
     update_success = res and update_success
 
-    if enable_license_key_secret:
-        update_success = update_success and integrations.auto_install_license_key(
-            session, tags
-        )
+    if input.enable_license_key_secret:
+        update_success = update_success and integrations.auto_install_license_key(input)
     else:
-        integrations.remove_license_key(session)
+        integrations.remove_license_key(input)
 
     if update_success:
         done("Update Complete")
